import 'dart:developer';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:collection/collection.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_widget_from_html_core/flutter_widget_from_html_core.dart';
import 'package:phoenix_widgets/phoenix_widgets.dart';
import 'package:url_launcher/url_launcher_string.dart';

class RichTextHandler extends StatelessWidget {
  const RichTextHandler({
    super.key,
    required this.content,
  });

  final String? content;

  @override
  Widget build(BuildContext context) {
    final List<String>? paragraphs =
        content?.replaceAll('/p><p', '/p>\n<p').split('\n');

    // Set proper sizes for content text and remove breaks.
    // this can be used to also pre-process and remove the justification of text
    for (var i = 0; i < (paragraphs?.length ?? 0); i++) {
      paragraphs?[i] = paragraphs[i].replaceAll('<br>', '');
    }

    // Content without Image Header + Blog Title
    final contentNoHeaderTitle = paragraphs?.join("</p>");
    return HtmlWidget(
      contentNoHeaderTitle ?? '',
      customStylesBuilder: (element) {
        if (element.localName == 'h3') {
          //? header H3
          return {
            'font-weight': '500',
            'font-size': '24px',
            'line-height': '140%',
            'color': '#212124;',
          };
        } else if (element.localName == 'h4') {
          //? header H4
          return {
            'font-weight': '500',
            'font-size': '20px',
            'line-height': '140%',
            'color': '#212124;',
          };
        } else if (element.localName == 'h2') {
          //? header H2
          return {
            'font-weight': '500',
            'font-size': '26px',
            'line-height': '140%',
            'color': '#212124;',
          };
        } else if (element.localName == 'h1') {
          //? header H1
          return {
            'font-weight': '1000',
            'font-size': '28px',
            'line-height': '140%',
            'color': '#212124;',
          };
        } else if (element.localName == 'a') {
          //? link a href
          return {
            'font-weight': '500',
            'color': '#67C1BF',
            'text-decoration': 'none',
          };
        } else if (element.localName == 'strong') {
          //? bold strong
          return {
            'font-weight': '1000',
          };
        } else if (element.localName == 'arabic' ||
            element.localName == 'sub') {
          //? sub was used to denote arabic in CS by the content-creators
          //? arabic
          return {
            'font-family': 'me_quran',
            'font-weight': '500',
            'font-size': '24px',
          };
        } else {
          //? regular text style
          return {
            'font-weight': 'normal',
            'font-size': '16px',
            'line-height': '150%',
            'color': '#4C4C50',
          };
        }
      },
      customWidgetBuilder: (element) {
        //? image element
        if (element.outerHtml.contains('figure') &&
            element.innerHtml.contains('img')) {
          final caption = element.nodes.firstWhereOrNull(
            (node) => node.text?.isNotEmpty ?? false,
          );

          final imageSrc = element.querySelector('img')?.attributes['src'];

          return Column(
            mainAxisSize: MainAxisSize.min,
            children: [
              ClipRRect(
                borderRadius: BorderRadius.circular(4),
                child: CachedNetworkImage(
                  imageUrl: imageSrc ?? '',
                  fit: BoxFit.cover,
                  errorWidget: (_, __, ___) => AspectRatio(
                    aspectRatio: 16 / 9,
                    child: Text(
                      'Image: ${element.attributes['alt'] ?? '...'}',
                    ),
                  ),
                ),
              ),
              if (caption != null) ...[
                const SizedBox(height: 4),
                Text(
                  caption.text ?? '...',
                ),
              ],
              const SizedBox(
                height: 8,
              ),
            ],
          );
        }
        // Display WebView video for each URL
        if (element.innerHtml.contains('iframe')) {
          final divElements = element.querySelectorAll('.item');
          final List<Map<String, String>> videosDetails = [];

          for (final divElement in divElements) {
            final videoTitle = divElement.text.trim();
            final videoSrc =
                divElement.querySelector('iframe')?.attributes['src'] ?? '';
            videosDetails.add({'videoTitle': videoTitle, 'videoSrc': videoSrc});
          }

          return Padding(
            padding: const EdgeInsets.symmetric(horizontal: 16),
            child: Column(
              children: [
                for (final Map<String, String> videoDetails
                    in videosDetails) ...[
                  Align(
                    alignment: Alignment.topLeft,
                    child: Text(
                      videoDetails['videoTitle'] ?? '',
                      style: Theme.of(context).textTheme.bodyLarge,
                      textAlign: TextAlign.start,
                    ),
                  ),
                  SizedBox(
                    height: 390,
                    child: PhoenixWebview(
                      title: videoDetails['videoTitle'] ?? '',
                      url: videoDetails['videoSrc'] ?? '',
                      showAppBar: false,
                    ),
                  ),
                  const SizedBox(height: 16),
                ],
              ],
            ),
          );
        } else {
          return null;
        }
      },
      onErrorBuilder: (context, element, error) {
        if (kDebugMode) {
          log(error);
          return Text('$element error: $error');
        }
        return null;
      },
      onTapUrl: (url) async {
        return launchUrlString(url);
      },
    );
  }
}
